@@ -2,12 +2,13 @@
 
 import click
 
+from SoftLayer.CLI.command import SLCommand as SLCommand
 from SoftLayer.CLI import environment
 from SoftLayer.CLI import helpers
 from SoftLayer.managers.vs_placement import PlacementManager as PlacementManager
 
 
-@click.command()
+@click.command(cls=SLCommand)
 @click.option('--name', type=click.STRING, required=True, prompt=True, help="Name for this new placement group.")
 @click.option('--backend_router', '-b', required=True, prompt=True,
               help="backendRouter, can be either the hostname or id.")
@@ -28,4 +29,4 @@ def cli(env, **args):
     }
 
     result = manager.create(placement_object)
-    click.secho("Successfully created placement group: ID: %s, Name: %s" % (result['id'], result['name']), fg='green')
+    click.secho(f"Successfully created placement group: ID: {result['id']}, Name: {result['name']}", fg='green')
