@@ -5,15 +5,15 @@
 
     :license: MIT, see License for more details.
 """
-
 import logging
-import SoftLayer
 
+from SoftLayer.exceptions import SoftLayerAPIError
+from SoftLayer.exceptions import SoftLayerError
 from SoftLayer.managers import ordering
 from SoftLayer import utils
 
 # Invalid names are ignored due to long method names and short argument names
-# pylint: disable=invalid-name, no-self-use
+# pylint: disable=invalid-name
 
 LOGGER = logging.getLogger(__name__)
 
@@ -171,8 +171,8 @@ class DedicatedHostManager(utils.IdentifierMixin, object):
         kwargs['iter'] = True
         return self.host.getGuests(id=host_id, **kwargs)
 
-    def list_instances(self, tags=None, cpus=None, memory=None, hostname=None,
-                       disk=None, datacenter=None, **kwargs):
+    def list_instances(self, tags=None, hostname=None,
+                       datacenter=None, order=None, owner=None):
         """Retrieve a list of all dedicated hosts on the account
 
         :param list tags: filter based on list of tags
@@ -185,19 +185,7 @@ class DedicatedHostManager(utils.IdentifierMixin, object):
         :returns: Returns a list of dictionaries representing the matching dedicated host.
 
         """
-        if 'mask' not in kwargs:
-            items = [
-                'id',
-                'name',
-                'cpuCount',
-                'diskCapacity',
-                'memoryCapacity',
-                'datacenter',
-                'guestCount',
-            ]
-            kwargs['mask'] = "mask[%s]" % ','.join(items)
-
-        _filter = utils.NestedDict(kwargs.get('filter') or {})
+        _filter = utils.NestedDict({})
         if tags:
             _filter['dedicatedHosts']['tagReferences']['tag']['name'] = {
                 'operation': 'in',
@@ -209,23 +197,22 @@ class DedicatedHostManager(utils.IdentifierMixin, object):
                 utils.query_filter(hostname)
             )
 
-        if cpus:
-            _filter['dedicatedHosts']['cpuCount'] = utils.query_filter(cpus)
-
-        if disk:
-            _filter['dedicatedHosts']['diskCapacity'] = (
-                utils.query_filter(disk))
-
-        if memory:
-            _filter['dedicatedHosts']['memoryCapacity'] = (
-                utils.query_filter(memory))
-
         if datacenter:
             _filter['dedicatedHosts']['datacenter']['name'] = (
                 utils.query_filter(datacenter))
 
-        kwargs['filter'] = _filter.to_dict()
-        return self.account.getDedicatedHosts(**kwargs)
+        if order:
+            _filter['dedicatedHosts']['billingItem']['orderItem']['order']['id'] = (
+                utils.query_filter(order))
+
+        if owner:
+            _filter['dedicatedHosts']['billingItem']['orderItem']['order']['userRecord']['username'] = (
+                utils.query_filter(owner))
+
+        object_filter = _filter.to_dict()
+        object_mask = "mask[id,name,createDate,cpuCount,diskCapacity,memoryCapacity,guestCount," \
+                      "datacenter,backendRouter,allocationStatus,billingItem[orderItem[order[userRecord]]]]"
+        return self.account.getDedicatedHosts(mask=object_mask, filter=object_filter)
 
     def get_host(self, host_id, **kwargs):
         """Get details about a dedicated host.
@@ -246,7 +233,7 @@ class DedicatedHostManager(utils.IdentifierMixin, object):
 
         """
         if 'mask' not in kwargs:
-            kwargs['mask'] = ('''
+            kwargs['mask'] = '''
                 id,
                 name,
                 cpuCount,
@@ -285,7 +272,7 @@ class DedicatedHostManager(utils.IdentifierMixin, object):
                     uuid
                 ],
                 guestCount
-            ''')
+            '''
 
         return self.host.getObject(id=host_id, **kwargs)
 
@@ -395,7 +382,7 @@ class DedicatedHostManager(utils.IdentifierMixin, object):
             if region['location']['location']['name'] == datacenter:
                 return region
 
-        raise SoftLayer.SoftLayerError("Could not find valid location for: '%s'" % datacenter)
+        raise SoftLayerError("Could not find valid location for: '%s'" % datacenter)
 
     def get_create_options(self):
         """Returns valid options for ordering a dedicated host."""
@@ -426,7 +413,7 @@ class DedicatedHostManager(utils.IdentifierMixin, object):
             if not price.get('locationGroupId'):
                 return price['id']
 
-        raise SoftLayer.SoftLayerError("Could not find valid price")
+        raise SoftLayerError("Could not find valid price")
 
     def _get_item(self, package, flavor):
         """Returns the item for ordering a dedicated host."""
@@ -435,7 +422,7 @@ class DedicatedHostManager(utils.IdentifierMixin, object):
             if item['keyName'] == flavor:
                 return item
 
-        raise SoftLayer.SoftLayerError("Could not find valid item for: '%s'" % flavor)
+        raise SoftLayerError("Could not find valid item for: '%s'" % flavor)
 
     def _get_backend_router(self, locations, item):
         """Returns valid router options for ordering a dedicated host."""
@@ -444,7 +431,9 @@ class DedicatedHostManager(utils.IdentifierMixin, object):
             hostname
         '''
         cpu_count = item['capacity']
-
+        mem_capacity = {}
+        disk_capacity = {}
+        gpuComponents = {}
         for capacity in item['bundleItems']:
             for category in capacity['categories']:
                 if category['categoryCode'] == 'dedicated_host_ram':
@@ -495,7 +484,7 @@ class DedicatedHostManager(utils.IdentifierMixin, object):
                     routers = self.host.getAvailableRouters(host, mask=mask)
                     return routers
 
-        raise SoftLayer.SoftLayerError("Could not find available routers")
+        raise SoftLayerError("Could not find available routers")
 
     def _get_default_router(self, routers, router_name=None):
         """Returns the default router for ordering a dedicated host."""
@@ -508,7 +497,7 @@ class DedicatedHostManager(utils.IdentifierMixin, object):
                 if router['hostname'] == router_name:
                     return router['id']
 
-        raise SoftLayer.SoftLayerError("Could not find valid default router")
+        raise SoftLayerError("Could not find valid default router")
 
     def get_router_options(self, datacenter=None, flavor=None):
         """Returns available backend routers for the dedicated host."""
@@ -524,7 +513,7 @@ class DedicatedHostManager(utils.IdentifierMixin, object):
         msg = 'Cancelled'
         try:
             self.guest.deleteObject(id=guest_id)
-        except SoftLayer.SoftLayerAPIError as e:
+        except SoftLayerAPIError as e:
             msg = 'Exception: ' + e.faultString
 
         return msg
