@@ -7,41 +7,68 @@
 """
 import collections
 import json
+import logging
+
+from SoftLayer.decoration import retry
 
 from SoftLayer import exceptions
 from SoftLayer import utils
 
 from SoftLayer.managers import event_log
 
+LOGGER = logging.getLogger(__name__)
+
+# pylint: disable=too-many-public-methods
+
 DEFAULT_SUBNET_MASK = ','.join(['hardware',
                                 'datacenter',
+                                'networkVlanId',
                                 'ipAddressCount',
                                 'virtualGuests',
-                                'networkVlan[id,networkSpace]'])
+                                'id',
+                                'networkIdentifier',
+                                'cidr',
+                                'subnetType',
+                                'gateway',
+                                'broadcastAddress',
+                                'usableIpAddressCount',
+                                'note',
+                                'tagReferences[tag]',
+                                'networkVlan[id, primaryRouter, networkSpace,fullyQualifiedName]',
+                                'addressSpace',
+                                'endPointIpAddress'
+                                ])
 DEFAULT_VLAN_MASK = ','.join([
     'firewallInterfaces',
     'hardwareCount',
     'primaryRouter[id, fullyQualifiedDomainName, datacenter]',
     'subnetCount',
+    'billingItem',
     'totalPrimaryIpAddressCount',
     'virtualGuestCount',
     'networkSpace',
+    'networkVlanFirewall[id,fullyQualifiedDomainName,primaryIpAddress]',
+    'attachedNetworkGateway[id,name,networkFirewall]',
+    'tagReferences[tag[name]]',
 ])
 DEFAULT_GET_VLAN_MASK = ','.join([
     'firewallInterfaces',
     'primaryRouter[id, fullyQualifiedDomainName, datacenter]',
     'totalPrimaryIpAddressCount',
     'networkSpace',
+    'billingItem',
     'hardware',
     'subnets',
     'virtualGuests',
+    'networkVlanFirewall[id,fullyQualifiedDomainName,primaryIpAddress]',
+    'attachedNetworkGateway[id,name,networkFirewall]',
 ])
 
 
 class NetworkManager(object):
-    """Manage SoftLayer network objects: VLANs, subnets, IPs and rwhois
+    """Manage SoftLayer network objects: VLANs, subnets and IPs
 
-    See product information here: http://www.softlayer.com/networking
+    See product information here: https://www.ibm.com/cloud/network
 
     :param SoftLayer.API.BaseClient client: the client instance
 
@@ -54,6 +81,7 @@ class NetworkManager(object):
         self.subnet = client['Network_Subnet']
         self.network_storage = self.client['Network_Storage']
         self.security_group = self.client['Network_SecurityGroup']
+        self.resolvers = [self.search_for_vlan]
 
     def add_global_ip(self, version=4, test_order=False):
         """Adds a global IP address to the account.
@@ -148,14 +176,14 @@ class NetworkManager(object):
         # item description.
         price_id = None
         quantity_str = str(quantity)
-        package_items = package.getItems(id=0)
+        package_items = package.getItems(id=0, mask='mask[prices[packageReferences[package[keyName]]]]')
         for item in package_items:
             category_code = utils.lookup(item, 'itemCategory', 'categoryCode')
             if all([category_code == category,
                     item.get('capacity') == quantity_str,
                     version == 4 or (version == 6 and
                                      desc in item['description'])]):
-                price_id = item['prices'][0]['id']
+                price_id = self.get_subnet_item_price(item, subnet_type, version)
                 break
 
         order = {
@@ -176,6 +204,24 @@ class NetworkManager(object):
         else:
             return self.client['Product_Order'].placeOrder(order)
 
+    @staticmethod
+    def get_subnet_item_price(item, subnet_type, version):
+        """Get the subnet specific item price id.
+
+        :param version: 4 for IPv4, 6 for IPv6.
+        :param subnet_type: Type of subnet to add: private, public, global,static.
+        :param item: Subnet item.
+        """
+        price_id = None
+        if version == 4 and subnet_type == 'static':
+            for item_price in item['prices']:
+                for package_reference in item_price['packageReferences']:
+                    if subnet_type.upper() in package_reference['package']['keyName']:
+                        price_id = item_price['id']
+        else:
+            price_id = item['prices'][0]['id']
+        return price_id
+
     def assign_global_ip(self, global_ip_id, target):
         """Assigns a global IP address to a specified target.
 
@@ -226,6 +272,22 @@ class NetworkManager(object):
         billing_id = subnet['billingItem']['id']
         return self.client['Billing_Item'].cancelService(id=billing_id)
 
+    def set_tags_subnet(self, subnet_id, tags):
+        """Tag a subnet by passing in one or more tags separated by a comma.
+
+        :param int subnet_id: The ID of the subnet.
+        :param string tags:	Comma separated list of tags.
+        """
+        return self.subnet.setTags(tags, id=subnet_id)
+
+    def edit_note_subnet(self, subnet_id, note):
+        """Edit the note for this subnet.
+
+        :param int subnet_id: The ID of the subnet.
+        :param string note:	The note.
+        """
+        return self.subnet.editNote(note, id=subnet_id)
+
     def create_securitygroup(self, name=None, description=None):
         """Creates a security group.
 
@@ -260,34 +322,6 @@ class NetworkManager(object):
         return self.security_group.detachNetworkComponents(component_ids,
                                                            id=group_id)
 
-    def edit_rwhois(self, abuse_email=None, address1=None, address2=None,
-                    city=None, company_name=None, country=None,
-                    first_name=None, last_name=None, postal_code=None,
-                    private_residence=None, state=None):
-        """Edit rwhois record."""
-        update = {}
-        for key, value in [('abuseEmail', abuse_email),
-                           ('address1', address1),
-                           ('address2', address2),
-                           ('city', city),
-                           ('companyName', company_name),
-                           ('country', country),
-                           ('firstName', first_name),
-                           ('lastName', last_name),
-                           ('privateResidenceFlag', private_residence),
-                           ('state', state),
-                           ('postalCode', postal_code)]:
-            if value is not None:
-                update[key] = value
-
-        # If there's anything to update, update it
-        if update:
-            rwhois = self.get_rwhois()
-            return self.client['Network_Subnet_Rwhois_Data'].editObject(
-                update, id=rwhois['id'])
-
-        return True
-
     def edit_securitygroup(self, group_id, name=None, description=None):
         """Edit security group details.
 
@@ -357,13 +391,6 @@ class NetworkManager(object):
         obj = self.client['Network_Subnet_IpAddress']
         return obj.getByIpAddress(ip_address, mask='hardware, virtualGuest')
 
-    def get_rwhois(self):
-        """Returns the RWhois information about the current account.
-
-        :returns: A dictionary containing the account's RWhois information.
-        """
-        return self.account.getRwhoisData()
-
     def get_securitygroup(self, group_id, **kwargs):
         """Returns the information about the given security group.
 
@@ -406,15 +433,21 @@ class NetworkManager(object):
 
         return self.subnet.getObject(id=subnet_id, **kwargs)
 
-    def get_vlan(self, vlan_id):
+    def get_vlan(self, vlan_id, mask=None):
         """Returns information about a single VLAN.
 
-        :param int id: The unique identifier for the VLAN
+        :param int vlan_id: The unique identifier for the VLAN
+        :param string mask: mask for request
         :returns: A dictionary containing a large amount of information about
                   the specified VLAN.
-
         """
-        return self.vlan.getObject(id=vlan_id, mask=DEFAULT_GET_VLAN_MASK)
+
+        if mask:
+            _mask = mask
+        else:
+            _mask = DEFAULT_GET_VLAN_MASK
+
+        return self.vlan.getObject(id=vlan_id, mask=_mask)
 
     def list_global_ips(self, version=None, identifier=None, **kwargs):
         """Returns a list of all global IP address records on the account.
@@ -474,9 +507,6 @@ class NetworkManager(object):
             _filter['subnets']['version'] = utils.query_filter(version)
         if subnet_type:
             _filter['subnets']['subnetType'] = utils.query_filter(subnet_type)
-        else:
-            # This filters out global IPs from the subnet listing.
-            _filter['subnets']['subnetType'] = {'operation': '!= GLOBAL_IP'}
         if network_space:
             _filter['subnets']['networkVlan']['networkSpace'] = (
                 utils.query_filter(network_space))
@@ -485,41 +515,37 @@ class NetworkManager(object):
         kwargs['iter'] = True
         return self.client.call('Account', 'getSubnets', **kwargs)
 
-    def list_vlans(self, datacenter=None, vlan_number=None, name=None, **kwargs):
+    def list_vlans(self, datacenter=None, vlan_number=None, name=None, limit=100, mask=None, _filter=None):
         """Display a list of all VLANs on the account.
 
         This provides a quick overview of all VLANs including information about
         data center residence and the number of devices attached.
 
-        :param string datacenter: If specified, the list will only contain
-                                    VLANs in the specified data center.
-        :param int vlan_number: If specified, the list will only contain the
-                                  VLAN matching this VLAN number.
-        :param int name: If specified, the list will only contain the
-                                  VLAN matching this VLAN name.
+        :param string datacenter: If specified, the list will only contain  VLANs in the specified data center.
+        :param int vlan_number: If specified, the list will only contain the VLAN matching this VLAN number.
+        :param int name: If specified, the list will only contain the VLAN matching this VLAN name.
         :param dict \\*\\*kwargs: response-level options (mask, limit, etc.)
 
         """
-        _filter = utils.NestedDict(kwargs.get('filter') or {})
+        _filter = utils.NestedDict(_filter or {})
+
+        _filter['networkVlans']['id'] = utils.query_filter_orderby()
 
         if vlan_number:
-            _filter['networkVlans']['vlanNumber'] = (
-                utils.query_filter(vlan_number))
+            _filter['networkVlans']['vlanNumber'] = utils.query_filter(vlan_number)
 
         if name:
             _filter['networkVlans']['name'] = utils.query_filter(name)
 
         if datacenter:
-            _filter['networkVlans']['primaryRouter']['datacenter']['name'] = (
-                utils.query_filter(datacenter))
+            _filter['networkVlans']['primaryRouter']['datacenter']['name'] = utils.query_filter(datacenter)
 
-        kwargs['filter'] = _filter.to_dict()
+        if mask is None:
+            mask = DEFAULT_VLAN_MASK
 
-        if 'mask' not in kwargs:
-            kwargs['mask'] = DEFAULT_VLAN_MASK
-
-        kwargs['iter'] = True
-        return self.account.getNetworkVlans(**kwargs)
+        # cf_call uses threads to get all results.
+        return self.client.cf_call('SoftLayer_Account', 'getNetworkVlans',
+                                   mask=mask, filter=_filter.to_dict(), limit=limit)
 
     def list_securitygroups(self, **kwargs):
         """List security groups."""
@@ -685,3 +711,185 @@ class NetworkManager(object):
         """
         result = self.network_storage.getObject(id=identifier, **kwargs)
         return result
+
+    def edit(self, instance_id, name=None, note=None, tags=None):
+        """Edit a vlan.
+
+        :param integer instance_id: the instance ID to edit.
+        :param string name: valid name.
+        :param string note: note about this particular vlan.
+        :param string tags: tags to set on the vlan as a comma separated list.
+                            Use the empty string to remove all tags.
+        :returns: bool -- True or an Exception
+        """
+
+        obj = {}
+
+        if tags is not None:
+            self.set_tags(tags, vlan_id=instance_id)
+
+        if name:
+            obj['name'] = name
+
+        if note:
+            obj['note'] = note
+
+        if not obj:
+            return True
+
+        return self.vlan.editObject(obj, id=instance_id)
+
+    @retry(logger=LOGGER)
+    def set_tags(self, tags, vlan_id):
+        """Sets tags on a vlan with a retry decorator
+
+        Just calls vlan.setTags, but if it fails from an APIError will retry.
+        """
+        self.vlan.setTags(tags, id=vlan_id)
+
+    def get_ip_by_address(self, ip_address):
+        """get the ip address object
+
+         :param string ip_address: the ip address to edit.
+         """
+        return self.client.call('SoftLayer_Network_Subnet_IpAddress', 'getByIpAddress', ip_address)
+
+    def set_subnet_ipddress_note(self, identifier, note):
+        """Set the ip address note of the subnet
+
+         :param integer identifier: the ip address ID to edit.
+          :param json note: the note to edit.
+        """
+        result = self.client.call('SoftLayer_Network_Subnet_IpAddress', 'editObject', note, id=identifier)
+        return result
+
+    def get_cancel_failure_reasons(self, identifier):
+        """get the reasons why we cannot cancel the VLAN.
+
+        :param integer identifier:  the instance ID
+        """
+        return self.vlan.getCancelFailureReasons(id=identifier)
+
+    def cancel_item(self, identifier, cancel_immediately,
+                    reason_cancel, customer_note):
+        """Cancel a billing item immediately, deleting all its data.
+
+        :param integer identifier: the instance ID to cancel
+        :param string reason_cancel: reason cancel
+        """
+        return self.client.call('SoftLayer_Billing_Item', 'cancelItem',
+                                True,
+                                cancel_immediately,
+                                reason_cancel,
+                                customer_note,
+                                id=identifier)
+
+    def get_pods(self, datacenter=None, mask=None):
+        """Calls SoftLayer_Network_Pod::getAllObjects()
+
+        returns list of all network pods and their routers.
+        """
+        _filter = {
+            'name': {
+                'operation': 'orderBy',
+                'options': [{'name': 'sort', 'value': ['DESC']}]
+            }
+        }
+        if datacenter:
+            _filter = {"datacenterName": {"operation": datacenter}}
+
+        _mask = None
+        if mask:
+            _mask = mask
+
+        return self.client.call('SoftLayer_Network_Pod', 'getAllObjects', mask=_mask, filter=_filter)
+
+    def get_list_datacenter(self):
+        """Calls SoftLayer_Location::getDatacenters()
+
+        returns all datacenter locations.
+        """
+        return self.client.call('SoftLayer_Location_Datacenter', 'getDatacenters')
+
+    def get_routers(self, identifier):
+        """Calls SoftLayer_Location::getRouters()
+
+        returns all routers locations.
+        """
+        return self.client.call('SoftLayer_Location_Datacenter', 'getHardwareRouters', id=identifier)
+
+    def get_closed_pods(self):
+        """Calls SoftLayer_Network_Pod::getAllObjects()
+
+        returns list of all closing network pods.
+        """
+        closing_filter = {
+            'capabilities': {
+                'operation': 'in',
+                'options': [{'name': 'data', 'value': ['CLOSURE_ANNOUNCED']}]
+            },
+            'name': {
+                'operation': 'orderBy',
+                'options': [{'name': 'sort', 'value': ['DESC']}]
+            }
+        }
+
+        mask = """mask[name, datacenterLongName, frontendRouterId, capabilities, datacenterId, backendRouterId,
+                backendRouterName, frontendRouterName]"""
+        return self.client.call('SoftLayer_Network_Pod', 'getAllObjects', mask=mask, filter=closing_filter)
+
+    def route(self, subnet_id, type_serv, target):
+        """Assigns a subnet to a specified target.
+
+        https://sldn.softlayer.com/reference/services/SoftLayer_Network_Subnet/route/
+        :param int subnet_id: The ID of the SoftLayer_Network_Subnet_IpAddress being routed
+        :param string type_serv: The type service to assign
+        :param string target: The instance to assign
+        """
+        return self.client.call('SoftLayer_Network_Subnet', 'route', type_serv, target, id=subnet_id, )
+
+    def get_datacenter(self, _filter=None, datacenter=None):
+        """Calls SoftLayer_Location::getDatacenters()
+
+        returns datacenter list.
+        """
+        if datacenter:
+            _filter = {"name": {"operation": datacenter}}
+
+        return self.client.call('SoftLayer_Location', 'getDatacenters', filter=_filter, limit=1)
+
+    def get_datacenter_by_keyname(self, keyname=None):
+        """Calls SoftLayer_Location::getDatacenters()
+
+        returns datacenter list.
+        """
+        mask = "mask[id,longName,name,regions[keyname,description]]"
+        _filter = {}
+        if keyname:
+            _filter = {"regions": {"keyname": {"operation": keyname}}}
+
+        result = self.client.call('SoftLayer_Location', 'getDatacenters', filter=_filter, limit=1, mask=mask)
+        if len(result) >= 1:
+            return result[0]
+        return {}
+
+    def clear_route(self, identifier):
+        """Calls SoftLayer_Network_Subnet::clearRoute()
+
+        returns  true or false.
+        """
+        return self.client.call('SoftLayer_Network_Subnet', 'clearRoute', id=identifier)
+
+    def search_for_vlan(self, vlan):
+        """Returns a list of matching VLAN objects.
+
+        :param string vlan: Could be either vlan name, number, id, or fully qualified name
+        :return list: List of SoftLayer_Network_Vlan objects
+        """
+
+        query = f"_objectType:SoftLayer_Network_Vlan {vlan}"
+        mask = "mask[resource(SoftLayer_Network_Vlan)[id,name,vlanNumber,fullyQualifiedName,networkSpace]]"
+
+        results = self.client.call('SoftLayer_Search', 'advancedSearch', query, mask=mask)
+        # This returns JUST the Network_Vlan information, none of the search information.
+        return [result.get('resource') for result in results]
