@@ -1,6 +1,6 @@
 """
     SoftLayer.storage_utils
-    ~~~~~~~~~~~~~~~
+    ~~~~~~~~~~~~~~~~~~~~~~~
     Utility functions used by File and Block Storage Managers
 
     :license: MIT, see LICENSE for more details.
@@ -19,19 +19,19 @@ ENDURANCE_TIERS = {
 }
 
 
-def populate_host_templates(host_templates,
-                            hardware_ids=None,
+def populate_host_templates(hardware_ids=None,
                             virtual_guest_ids=None,
                             ip_address_ids=None,
                             subnet_ids=None):
-    """Populate the given host_templates array with the IDs provided
+    """Returns a populated array with the IDs provided
 
-    :param host_templates: The array to which host templates will be added
     :param hardware_ids: A List of SoftLayer_Hardware ids
     :param virtual_guest_ids: A List of SoftLayer_Virtual_Guest ids
     :param ip_address_ids: A List of SoftLayer_Network_Subnet_IpAddress ids
     :param subnet_ids: A List of SoftLayer_Network_Subnet ids
+    :return: array of objects formatted for allowAccessFromHostList
     """
+    host_templates = []
     if hardware_ids is not None:
         for hardware_id in hardware_ids:
             host_templates.append({
@@ -59,6 +59,7 @@ def populate_host_templates(host_templates,
                 'objectType': 'SoftLayer_Network_Subnet',
                 'id': subnet_id
             })
+    return host_templates
 
 
 def get_package(manager, category_code):
@@ -72,7 +73,7 @@ def get_package(manager, category_code):
     _filter = utils.NestedDict({})
     _filter['categories']['categoryCode'] = (
         utils.query_filter(category_code))
-    _filter['statusCode'] = (utils.query_filter('ACTIVE'))
+    _filter['statusCode'] = utils.query_filter('ACTIVE')
 
     packages = manager.client.call(
         'Product_Package', 'getAllObjects',
@@ -238,7 +239,7 @@ def find_saas_endurance_space_price(package, size, tier_level):
     """
     if tier_level != 0.25:
         tier_level = int(tier_level)
-    key_name = 'STORAGE_SPACE_FOR_{0}_IOPS_PER_GB'.format(tier_level)
+    key_name = f'STORAGE_SPACE_FOR_{tier_level}_IOPS_PER_GB'
     key_name = key_name.replace(".", "_")
     for item in package['items']:
         if key_name not in item['keyName']:
@@ -306,7 +307,7 @@ def find_saas_perform_space_price(package, size):
         if size < capacity_minimum or size > capacity_maximum:
             continue
 
-        key_name = '{0}_{1}_GBS'.format(capacity_minimum, capacity_maximum)
+        key_name = f'{capacity_minimum}_{capacity_maximum}_GBS'
         if item['keyName'] != key_name:
             continue
         price_id = _find_price_id(item['prices'], 'performance_storage_space')
@@ -418,11 +419,10 @@ def find_snapshot_schedule_id(volume, snapshot_schedule_keyname):
             if schedule['type']['keyname'] == snapshot_schedule_keyname:
                 return schedule['id']
 
-    raise ValueError("The given snapshot schedule ID was not found for "
-                     "the given storage volume")
+    raise ValueError("The given snapshot schedule ID was not found for the given storage volume")
 
 
-def prepare_snapshot_order_object(manager, volume, capacity, tier, upgrade):
+def prepare_snapshot_order_object(manager, volume, capacity, tier, upgrade, iops):
     """Prepare the snapshot space order object for the placeOrder() method
 
     :param manager: The File or Block manager calling this function
@@ -435,8 +435,7 @@ def prepare_snapshot_order_object(manager, volume, capacity, tier, upgrade):
     """
     # Ensure the storage volume has not been cancelled
     if 'billingItem' not in volume:
-        raise exceptions.SoftLayerError(
-            'This volume has been cancelled; unable to order snapshot space')
+        raise exceptions.SoftLayerError('This volume has been cancelled; unable to order snapshot space')
 
     # Determine and validate the storage volume's billing item category
     billing_item_category_code = volume['billingItem']['categoryCode']
@@ -465,7 +464,7 @@ def prepare_snapshot_order_object(manager, volume, capacity, tier, upgrade):
                 raise exceptions.SoftLayerError(
                     "Snapshot space cannot be ordered for this performance "
                     "volume since it does not support Encryption at Rest.")
-            iops = int(volume['provisionedIops'])
+
             prices = [find_saas_snapshot_space_price(
                 package, capacity, iops=iops)]
         else:
@@ -532,10 +531,9 @@ def prepare_volume_order_object(manager, storage_type, location, size,
     # Find the ID for the requested location
     try:
         location_id = get_location_id(manager, location)
-    except ValueError:
-        raise exceptions.SoftLayerError(
-            "Invalid datacenter name specified. "
-            "Please provide the lower case short name (e.g.: dal09)")
+    except ValueError as ex:
+        message = "Invalid datacenter name specified. Please provide the lower case short name (e.g.: dal09)"
+        raise exceptions.SoftLayerError(message) from ex
 
     # Determine the category code to use for the order (and product package)
     order_type_is_saas, order_category_code = _get_order_type_and_category(
@@ -645,7 +643,7 @@ def _get_order_type_and_category(service_offering, storage_type, volume_type):
 
 
 def prepare_replicant_order_object(manager, snapshot_schedule, location,
-                                   tier, volume, volume_type):
+                                   tier, volume, volume_type, iops):
     """Prepare the order object which is submitted to the placeOrder() method
 
     :param manager: The File or Block manager calling this function
@@ -675,10 +673,9 @@ def prepare_replicant_order_object(manager, snapshot_schedule, location,
     # Find the ID for the requested location
     try:
         location_id = get_location_id(manager, location)
-    except ValueError:
-        raise exceptions.SoftLayerError(
-            "Invalid datacenter name specified. "
-            "Please provide the lower case short name (e.g.: dal09)")
+    except ValueError as ex:
+        message = "Invalid datacenter name specified. Please provide the lower case short name (e.g.: dal09)"
+        raise exceptions.SoftLayerError(message) from ex
 
     # Get sizes and properties needed for the order
     volume_size = int(volume['capacityGb'])
@@ -731,7 +728,7 @@ def prepare_replicant_order_object(manager, snapshot_schedule, location,
                     "A replica volume cannot be ordered for this performance "
                     "volume since it does not support Encryption at Rest.")
             volume_is_performance = True
-            iops = int(volume['provisionedIops'])
+
             prices = [
                 find_price_by_category(package, billing_item_category_code),
                 find_price_by_category(package, 'storage_' + volume_type),
@@ -788,7 +785,8 @@ def prepare_replicant_order_object(manager, snapshot_schedule, location,
 
 def prepare_duplicate_order_object(manager, origin_volume, iops, tier,
                                    duplicate_size, duplicate_snapshot_size,
-                                   volume_type, hourly_billing_flag=False):
+                                   volume_type, hourly_billing_flag=False,
+                                   dependent_duplicate=False):
     """Prepare the duplicate order to submit to SoftLayer_Product::placeOrder()
 
     :param manager: The File or Block manager calling this function
@@ -799,6 +797,8 @@ def prepare_duplicate_order_object(manager, origin_volume, iops, tier,
     :param duplicate_snapshot_size: The size for the duplicate snapshot space
     :param volume_type: The type of the origin volume ('file' or 'block')
     :param hourly_billing_flag: Billing type, monthly (False) or hourly (True)
+    :param dependent_duplicate: Duplicate type, normal (False) or dependent
+        duplicate (True)
     :return: Returns the order object to be passed to the
              placeOrder() method of the Product_Order service
     """
@@ -903,6 +903,9 @@ def prepare_duplicate_order_object(manager, origin_volume, iops, tier,
     if volume_is_performance:
         duplicate_order['iops'] = iops
 
+    if dependent_duplicate:
+        duplicate_order['isDependentDuplicateFlag'] = 1
+
     return duplicate_order
 
 
@@ -985,6 +988,15 @@ def prepare_modify_order_object(manager, volume, new_iops, new_tier, new_size):
     return modify_order
 
 
+def block_or_file(storage_type_keyname):
+    """returns either 'block' or 'file'
+
+    :param storage_type_keyname: the Network_Storage['storageType']['keyName']
+    :returns: 'block' or 'file'
+    """
+    return 'block' if 'BLOCK_STORAGE' in storage_type_keyname else 'file'
+
+
 def _has_category(categories, category_code):
     return any(
         True
