@@ -13,18 +13,28 @@ import warnings
 
 from SoftLayer.decoration import retry
 from SoftLayer import exceptions
+from SoftLayer.exceptions import SoftLayerError
+from SoftLayer.managers.hardware import _get_preset_cost
+from SoftLayer.managers.hardware import get_item_price
 from SoftLayer.managers import ordering
 from SoftLayer import utils
 
 LOGGER = logging.getLogger(__name__)
 
-# pylint: disable=no-self-use,too-many-lines
+EXTRA_CATEGORIES = ['pri_ipv6_addresses',
+                    'static_ipv6_addresses',
+                    'sec_ip_addresses',
+                    'trusted_platform_module',
+                    'software_guard_extensions']
+
+
+# pylint: disable=,too-many-lines
 
 
 class VSManager(utils.IdentifierMixin, object):
     """Manages SoftLayer Virtual Servers.
 
-    See product information here: http://www.softlayer.com/virtual-servers
+    See product information here: https://www.ibm.com/cloud/virtual-servers
 
     Example::
 
@@ -51,6 +61,7 @@ class VSManager(utils.IdentifierMixin, object):
         self.account = client['Account']
         self.guest = client['Virtual_Guest']
         self.package_svc = client['Product_Package']
+        self.storage_iscsi = client['SoftLayer_Network_Storage_Iscsi']
         self.resolvers = [self._get_ids_from_ip, self._get_ids_from_hostname]
         if ordering_manager is None:
             self.ordering_manager = ordering.OrderingManager(client)
@@ -123,7 +134,7 @@ class VSManager(utils.IdentifierMixin, object):
         if tags:
             _filter['virtualGuests']['tagReferences']['tag']['name'] = {
                 'operation': 'in',
-                'options': [{'name': 'data', 'value': tags}],
+                'options': [{'name': 'data', 'value': list(tags)}],
             }
 
         if cpus:
@@ -216,7 +227,7 @@ class VSManager(utils.IdentifierMixin, object):
                 'maxMemory,'
                 'datacenter,'
                 'activeTransaction[id, transactionStatus[friendlyName,name]],'
-                'lastTransaction[transactionStatus],'
+                'lastTransaction[transactionStatus,modifyDate,transactionGroup[name]],'
                 'lastOperatingSystemReload.id,'
                 'blockDevices,'
                 'blockDeviceTemplateGroup[id, name, globalIdentifier],'
@@ -234,22 +245,29 @@ class VSManager(utils.IdentifierMixin, object):
                 'userData,'
                 '''billingItem[id,nextInvoiceTotalRecurringAmount,
                                package[id,keyName],
-                               children[categoryCode,nextInvoiceTotalRecurringAmount],
+                               children[description,categoryCode,nextInvoiceTotalRecurringAmount],
+                               nextInvoiceChildren[description,
+                                                   categoryCode,
+                                                   recurringFee,
+                                                   nextInvoiceTotalRecurringAmount],
                                orderItem[id,
                                          order.userRecord[username],
                                          preset.keyName]],'''
                 'tagReferences[id,tag[name,id]],'
                 'networkVlans[id,vlanNumber,networkSpace],'
                 'dedicatedHost.id,'
+                'monitoringServiceComponent,networkMonitors[queryType,lastResult,responseAction],'
                 'placementGroupId'
             )
 
         return self.guest.getObject(id=instance_id, **kwargs)
 
     @retry(logger=LOGGER)
-    def get_create_options(self):
+    def get_create_options(self, vsi_type="PUBLIC_CLOUD_SERVER", datacenter=None):
         """Retrieves the available options for creating a VS.
 
+        :param string vsi_type: vs keyName.
+        :param string datacenter: short name, like dal09
         :returns: A dictionary of creation options.
 
         Example::
@@ -258,7 +276,147 @@ class VSManager(utils.IdentifierMixin, object):
             options = mgr.get_create_options()
             print(options)
         """
-        return self.guest.getCreateObjectOptions()
+
+        # TRANSIENT_CLOUD_SERVER
+        # SUSPEND_CLOUD_SERVER
+        package = self._get_package(vsi_type)
+
+        location_group_id = None
+        if datacenter:
+            _filter = {"name": {"operation": datacenter}}
+            _mask = "mask[priceGroups]"
+            dc_details = self.client.call('SoftLayer_Location', 'getDatacenters', mask=_mask, filter=_filter, limit=1)
+            if not dc_details:
+                raise SoftLayerError(f"Unable to find a datacenter named {datacenter}")
+            # A DC will have several price groups, no good way to deal with this other than checking each.
+            # An item should only belong to one type of price group.
+            for group in dc_details[0].get('priceGroups', []):
+                # We only care about SOME of the priceGroups, which all SHOULD start with `Location Group X`
+                # Hopefully this never changes....
+                if group.get('description').startswith('Location'):
+                    location_group_id = group.get('id')
+
+        # Locations
+        locations = []
+        for region in package['regions']:
+            if datacenter is None or datacenter == region['location']['location']['name']:
+                locations.append({
+                    'name': region['location']['location']['longName'],
+                    'key': region['location']['location']['name'],
+                })
+
+        operating_systems = []
+        database = []
+        port_speeds = []
+        guest_core = []
+        local_disk = []
+        extras = []
+        ram = []
+
+        sizes = []
+        for preset in package['activePresets'] + package['accountRestrictedActivePresets']:
+            sizes.append({
+                'name': preset['description'],
+                'key': preset['keyName'],
+                'hourlyRecurringFee': _get_preset_cost(preset, package['items'], 'hourly', location_group_id),
+                'recurringFee': _get_preset_cost(preset, package['items'], 'monthly', location_group_id)
+            })
+
+        for item in package['items']:
+            category = item['itemCategory']['categoryCode']
+            # Operating systems
+            if category == 'os':
+                operating_systems.append({
+                    'name': item['softwareDescription']['longDescription'],
+                    'key': item['keyName'],
+                    'referenceCode': item['softwareDescription']['referenceCode'],
+                    'prices': get_item_price(item['prices'], location_group_id)
+                })
+            # database
+            elif category == 'database':
+                database.append({
+                    'name': item['description'],
+                    'key': item['keyName'],
+                    'prices': get_item_price(item['prices'], location_group_id)
+                })
+
+            elif category == 'port_speed':
+                port_speeds.append({
+                    'name': item['description'],
+                    'speed': item['capacity'],
+                    'key': item['keyName'],
+                    'prices': get_item_price(item['prices'], location_group_id)
+                })
+
+            elif category == 'guest_core':
+                guest_core.append({
+                    'name': item['description'],
+                    'capacity': item['capacity'],
+                    'key': item['keyName'],
+                    'prices': get_item_price(item['prices'], location_group_id)
+                })
+
+            elif category == 'ram':
+                ram.append({
+                    'name': item['description'],
+                    'capacity': item['capacity'],
+                    'key': item['keyName'],
+                    'prices': get_item_price(item['prices'], location_group_id)
+                })
+
+            elif 'guest_disk' in category:
+                local_disk.append({
+                    'name': item['description'],
+                    'capacity': item['capacity'],
+                    'key': item['keyName'],
+                    'disk': category,
+                    'prices': get_item_price(item['prices'], location_group_id)
+                })
+            # Extras
+            elif category in EXTRA_CATEGORIES:
+                extras.append({
+                    'name': item['description'],
+                    'key': item['keyName'],
+                    'prices': get_item_price(item['prices'], location_group_id)
+                })
+
+        return {
+            'locations': locations,
+            'ram': ram,
+            'database': database,
+            'operating_systems': operating_systems,
+            'guest_core': guest_core,
+            'port_speed': port_speeds,
+            'guest_disk': local_disk,
+            'sizes': sizes,
+            'extras': extras,
+        }
+
+    @retry(logger=LOGGER)
+    def _get_package(self, package_keyname):
+        """Get the package related to simple vs ordering.
+
+        :param string package_keyname: Virtual Server package keyName.
+        """
+        items_mask = 'mask[id,keyName,capacity,description,attributes[id,attributeTypeKeyName],' \
+                     'itemCategory[id,categoryCode],softwareDescription[id,referenceCode,longDescription],' \
+                     'prices[categories]]'
+        # The preset prices list will only have default prices. The prices->item->prices will have location specific
+        presets_mask = 'mask[prices]'
+        region_mask = 'location[location[priceGroups]]'
+        package = {'items': [], 'activePresets': [], 'accountRestrictedActivePresets': [], 'regions': []}
+        package_info = self.ordering_manager.get_package_by_key(package_keyname, mask="mask[id]")
+
+        package['items'] = self.client.call('SoftLayer_Product_Package', 'getItems',
+                                            id=package_info.get('id'), mask=items_mask)
+        package['activePresets'] = self.client.call('SoftLayer_Product_Package', 'getActivePresets',
+                                                    id=package_info.get('id'), mask=presets_mask)
+        package['accountRestrictedActivePresets'] = self.client.call('SoftLayer_Product_Package',
+                                                                     'getAccountRestrictedActivePresets',
+                                                                     id=package_info.get('id'), mask=presets_mask)
+        package['regions'] = self.client.call('SoftLayer_Product_Package', 'getRegions',
+                                              id=package_info.get('id'), mask=region_mask)
+        return package
 
     def cancel_instance(self, instance_id):
         """Cancel an instance immediately, deleting all its data.
@@ -380,10 +538,11 @@ class VSManager(utils.IdentifierMixin, object):
         if datacenter:
             data["datacenter"] = {"name": datacenter}
 
-        if private_vlan or public_vlan or private_subnet or public_subnet:
-            network_components = self._create_network_components(public_vlan, private_vlan,
-                                                                 private_subnet, public_subnet)
-            data.update(network_components)
+        network_components = self._create_network_components(public_vlan, private_vlan,
+                                                             private_subnet, public_subnet,
+                                                             kwargs.get('private_router'),
+                                                             kwargs.get('public_router'))
+        data.update(network_components)
 
         if public_security_groups:
             secgroups = [{'securityGroup': {'id': int(sg)}}
@@ -428,8 +587,17 @@ class VSManager(utils.IdentifierMixin, object):
 
     def _create_network_components(
             self, public_vlan=None, private_vlan=None,
-            private_subnet=None, public_subnet=None):
+            private_subnet=None, public_subnet=None,
+            private_router=None, public_router=None):
         parameters = {}
+        if any([private_router, public_router]) and any([private_vlan, public_vlan, private_subnet, public_subnet]):
+            raise exceptions.SoftLayerError("You have to select network vlan or network vlan with a subnet or "
+                                            "only router, not all options")
+
+        if private_router:
+            parameters['primaryBackendNetworkComponent'] = {"router": {"id": int(private_router)}}
+        if public_router:
+            parameters['primaryNetworkComponent'] = {"router": {"id": int(public_router)}}
         if private_vlan:
             parameters['primaryBackendNetworkComponent'] = {"networkVlan": {"id": int(private_vlan)}}
         if public_vlan:
@@ -532,7 +700,21 @@ class VSManager(utils.IdentifierMixin, object):
         kwargs.pop('tags', None)
         create_options = self._generate_create_dict(**kwargs)
         template = self.guest.generateOrderTemplate(create_options)
-        if 'private_subnet' in kwargs or 'public_subnet' in kwargs:
+        if kwargs.get('public_router') or kwargs.get('private_router'):
+            if kwargs.get('private_vlan') or kwargs.get('public_vlan') or kwargs.get('private_subnet') \
+                    or kwargs.get('public_subnet'):
+                raise exceptions.SoftLayerError("You have to select network vlan or network vlan with a subnet or "
+                                                "only router, not all options")
+            vsi = template['virtualGuests'][0]
+            network_components = self._create_network_components(kwargs.get('public_vlan', None),
+                                                                 kwargs.get('private_vlan', None),
+                                                                 kwargs.get('private_subnet', None),
+                                                                 kwargs.get('public_subnet', None),
+                                                                 kwargs.get('private_router', None),
+                                                                 kwargs.get('public_router', None))
+            vsi.update(network_components)
+
+        if kwargs.get('private_subnet') or kwargs.get('public_subnet'):
             vsi = template['virtualGuests'][0]
             network_components = self._create_network_components(kwargs.get('public_vlan', None),
                                                                  kwargs.get('private_vlan', None),
@@ -814,10 +996,10 @@ class VSManager(utils.IdentifierMixin, object):
 
             disks_to_capture.append(block_device)
 
-        return self.guest.createArchiveTransaction(
+        return self.guest.createArchiveTemplate(
             name, disks_to_capture, notes, id=instance_id)
 
-    def upgrade(self, instance_id, cpus=None, memory=None, nic_speed=None, public=True, preset=None):
+    def upgrade(self, instance_id, cpus=None, memory=None, nic_speed=None, public=True, preset=None, disk=None):
         """Upgrades a VS instance.
 
         Example::
@@ -861,21 +1043,50 @@ class VSManager(utils.IdentifierMixin, object):
             'virtualGuests': [{'id': int(instance_id)}],
         }
 
+        if disk:
+            disk_number = 0
+            vsi_disk = self.get_instance(instance_id)
+            for item in vsi_disk.get('billingItem').get('children'):
+                if 'guest_disk' in item.get('categoryCode'):
+                    disk_number = max(disk_number, int("".join(filter(str.isdigit, item.get('categoryCode')))))
+            for disk_guest in disk:
+                if disk_guest.get('number') > 0:
+                    disk_number = disk_guest.get('number')
+                else:
+                    disk_number = disk_number + 1
+                price_id = self._get_price_id_for_upgrade_option(upgrade_prices,
+                                                                 'disk',
+                                                                 disk_guest.get('capacity'),
+                                                                 public)
+
+                if price_id is None:
+                    error = f"Unable to find disk option with value {disk_guest.get('capacity')}"
+                    raise exceptions.SoftLayerAPIError(500, error)
+
+                category_id = self.get_disk_category_id_by_disk_number(disk_guest.get('capacity'), disk_number)
+                if category_id is None:
+                    raise exceptions.SoftLayerError('Invalid disk number to this disk capacity')
+                category = {'categories': [{
+                    'id': category_id,
+                    'complexType': "SoftLayer_Product_Item_Category"}],
+                    'complexType': 'SoftLayer_Product_Item_Price',
+                    'id': price_id}
+
+                prices.append(category)
+            order['prices'] = prices
+
         for option, value in data.items():
             if not value:
                 continue
-            price_id = self._get_price_id_for_upgrade_option(upgrade_prices,
-                                                             option,
-                                                             value,
-                                                             public)
+            price_id = self._get_price_id_for_upgrade_option(upgrade_prices, option, value, public)
             if not price_id:
                 # Every option provided is expected to have a price
                 raise exceptions.SoftLayerError(
                     "Unable to find %s option with value %s" % (option, value))
 
             prices.append({'id': price_id})
-        order['prices'] = prices
 
+            order['prices'] = prices
         if preset is not None:
             vs_object = self.get_instance(instance_id)['billingItem']['package']
             order['presetId'] = self.ordering_manager.get_preset_by_key(vs_object['keyName'], preset)['id']
@@ -885,6 +1096,43 @@ class VSManager(utils.IdentifierMixin, object):
             return True
         return False
 
+    def get_disk_category_id_by_disk_number(self, capacity, disk_number):
+        """Uses Product_Package::getItemPrices to get all disk items with its categories and
+
+        disk_key_names dictionary to convert disk numbers (int) to ordinal numbers (string)
+        """
+        disk_key_names = {
+            1: "First Disk",
+            2: "Second Disk",
+            3: "Third Disk",
+            4: "Fourth Disk",
+            5: "Fifth Disk",
+        }
+        category_to_request = disk_key_names.get(disk_number)
+        if category_to_request is None:
+            return None
+
+        key_name = "*= GUEST_DISK_" + str(capacity) + "_GB_SAN"
+        object_filter = {
+            "itemPrices": {
+                "locationGroupId": {"operation": "is null"},
+                "item": {
+                    "keyName": {"operation": key_name}
+                }
+            }
+        }
+        mask = "mask[id,locationGroupId,item[capacity,description,keyName],categories]"
+        disk_items = self.client.call(
+            'SoftLayer_Product_Package', 'getItemPrices', id=46, mask=mask, filter=object_filter
+        )
+        category_id = None
+        for disk_item in disk_items:
+            for category in disk_item.get('categories'):
+                if category.get('name') == category_to_request:
+                    category_id = category.get('id')
+                    break
+        return category_id
+
     def order_guest(self, guest_object, test=False):
         """Uses Product_Order::placeOrder to create a virtual guest.
 
@@ -993,9 +1241,11 @@ class VSManager(utils.IdentifierMixin, object):
         option_category = {
             'memory': 'ram',
             'cpus': 'guest_core',
-            'nic_speed': 'port_speed'
+            'nic_speed': 'port_speed',
+            'disk': 'guest_disk'
         }
         category_code = option_category.get(option)
+
         for price in upgrade_prices:
             if price.get('categories') is None or price.get('item') is None:
                 continue
@@ -1005,6 +1255,11 @@ class VSManager(utils.IdentifierMixin, object):
                           or product.get('units') == 'DEDICATED_CORE')
 
             for category in price.get('categories'):
+                if option == 'disk':
+                    if (category_code == (''.join([i for i in category.get('categoryCode') if not i.isdigit()]))
+                            and str(product.get('capacity')) == str(value)):
+                        return price.get('id')
+
                 if not (category.get('categoryCode') == category_code
                         and str(product.get('capacity')) == str(value)):
                     continue
@@ -1099,7 +1354,7 @@ class VSManager(utils.IdentifierMixin, object):
         }
         category_code = option_category[option]
         for item in package_items:
-            is_private = (item.get('units') == 'PRIVATE_CORE')
+            is_private = item.get('units') == 'PRIVATE_CORE'
             for price in item['prices']:
                 if 'locationGroupId' in price and price['locationGroupId']:
                     # Skip location based prices
@@ -1123,3 +1378,128 @@ class VSManager(utils.IdentifierMixin, object):
                             return price['id']
                     else:
                         return price['id']
+
+    def get_storage_details(self, instance_id, nas_type):
+        """Returns the virtual server attached network storage.
+
+        :param int instance_id: Id of the virtual server
+        :param nas_type: storage type.
+        """
+        mask = 'mask[id,username,capacityGb,notes,serviceResourceBackendIpAddress,' \
+               'allowedVirtualGuests[id,datacenter]]'
+        return self.guest.getAttachedNetworkStorages(nas_type, mask=mask, id=instance_id)
+
+    def get_storage_credentials(self, instance_id):
+        """Returns the virtual server storage credentials.
+
+        :param int instance_id: Id of the virtual server
+        """
+        mask = 'mask[credential]'
+        return self.guest.getAllowedHost(mask=mask, id=instance_id)
+
+    def get_portable_storage(self, instance_id):
+        """Returns the virtual server portable storage.
+
+        :param int instance_id: Id of the virtual server
+        """
+        object_filter = {"portableStorageVolumes": {"blockDevices": {"guest": {"id": {"operation": instance_id}}}}}
+        return self.account.getPortableStorageVolumes(filter=object_filter)
+
+    def get_local_disks(self, instance_id):
+        """Returns the virtual server local disks.
+
+        :param int instance_id: Id of the virtual server
+        """
+        mask = 'mask[diskImage]'
+        return self.guest.getBlockDevices(mask=mask, id=instance_id)
+
+    def migrate(self, instance_id):
+        """Calls SoftLayer_Virtual_Guest::migrate
+
+        Only actually does anything if the virtual server requires a migration.
+        Will return an exception otherwise.
+
+        :param int instance_id: Id of the virtual server
+        """
+        return self.guest.migrate(id=instance_id)
+
+    def migrate_dedicated(self, instance_id, host_id):
+        """Calls SoftLayer_Virtual_Guest::migrate
+
+        Only actually does anything if the virtual server requires a migration.
+        Will return an exception otherwise.
+
+        :param int instance_id: Id of the virtual server
+        """
+        return self.guest.migrateDedicatedHost(host_id, id=instance_id)
+
+    def authorize_storage(self, vs_id, username_storage):
+        """Authorize File or Block Storage to a Virtual Server.
+
+        :param int vs_id: Virtual server id.
+        :param string username_storage: Storage username.
+
+        :return: bool.
+        """
+        _filter = {"networkStorage": {"username": {"operation": username_storage}}}
+
+        storage_result = self.client.call('Account', 'getNetworkStorage', filter=_filter)
+
+        if len(storage_result) == 0:
+            raise SoftLayerError("The Storage with username: %s was not found, please"
+                                 " enter a valid storage username" % username_storage)
+
+        storage_template = [
+            {
+                "id": storage_result[0]['id'],
+                "username": username_storage
+            }
+        ]
+
+        result = self.client.call('SoftLayer_Virtual_Guest', 'allowAccessToNetworkStorageList',
+                                  storage_template, id=vs_id)
+
+        return result
+
+    def attach_portable_storage(self, vs_id, portable_id):
+        """Attach portal storage to a Virtual Server.
+
+        :param int vs_id: Virtual server id.
+        :param int portable_id: Portal storage id.
+
+        :return: SoftLayer_Provisioning_Version1_Transaction.
+        """
+        disk_id = portable_id
+        result = self.client.call('SoftLayer_Virtual_Guest', 'attachDiskImage',
+                                  disk_id, id=vs_id)
+
+        return result
+
+    def browser_access_log(self, identifier):
+        """A virtual guest’s browser access logs.
+
+        :param int identifier: Virtual server id.
+
+
+        :return: SoftLayer_Virtual_BrowserConsoleAccessLog.
+        """
+        mask = 'createDate,eventType,id,message,sourceIp,sourcePort,username'
+        return self.client.call('SoftLayer_Virtual_Guest', 'getBrowserConsoleAccessLogs', mask=mask, id=identifier)
+
+    def get_notifications(self, vs_id):
+        """Returns all virtual notifications."""
+        return self.client.call('SoftLayer_User_Customer_Notification_Virtual_Guest', 'findByGuestId', vs_id)
+
+    def add_notification(self, virtual_id, user_id):
+        """Create a user virtual notification entry"""
+
+        template = {"guestId": virtual_id, "userId": user_id}
+        mask = 'user'
+        return self.client.call('SoftLayer_User_Customer_Notification_Virtual_Guest',
+                                'createObject', template, mask=mask)
+
+    def remove_notification(self, identifier):
+        """Remove a user vs notification entry"""
+
+        template = [{'id': identifier}]
+        return self.client.call('SoftLayer_User_Customer_Notification_Virtual_Guest', 'deleteObjects', template)
