@@ -7,6 +7,8 @@ import json
 import sys
 import tempfile
 
+from unittest import mock as mock
+
 from SoftLayer.CLI import exceptions
 from SoftLayer import testing
 
@@ -45,6 +47,46 @@ class OrderTests(testing.TestCase):
         self.assertIn('testing', result.output)
         self.assertIn('item2', result.output)
 
+    def test_item_list_prices(self):
+        result = self.run_command(['order', 'item-list', 'package', '--prices'])
+
+        self.assert_no_fail(result)
+        output = json.loads(result.output)
+        self.assertEqual(output[0][0]['Price Id'], 1007)
+        self.assertEqual(output[0][1]['Restriction'], '- - - -')
+        self.assertEqual(output[0][1]['KeyName'], 'KeyName015')
+        self.assert_called_with('SoftLayer_Product_Package', 'getItems')
+
+    def test_item_list_location_keyname(self):
+        result = self.run_command(['order', 'item-list', 'package', '--prices', 'DALLAS13', ])
+
+        self.assert_no_fail(result)
+        output = json.loads(result.output)
+        self.assertEqual(output[0][0]['Hourly'], 0.0)
+        self.assertEqual(output[0][1]['KeyName'], 'KeyName015')
+        self.assertEqual(output[0][1]['Price Id'], 1144)
+        self.assert_called_with('SoftLayer_Product_Package', 'getItemPrices')
+
+    def test_item_list_location_name(self):
+        result = self.run_command(['order', 'item-list', 'package', '--prices', 'dal13', ])
+
+        self.assert_no_fail(result)
+        output = json.loads(result.output)
+        self.assertEqual(output[0][0]['Hourly'], 0.0)
+        self.assertEqual(output[0][1]['KeyName'], 'KeyName015')
+        self.assertEqual(output[0][1]['Price Id'], 1144)
+        self.assert_called_with('SoftLayer_Product_Package', 'getItemPrices')
+
+    def test_item_list_category_keyword(self):
+        result = self.run_command(['order', 'item-list', 'package', '--prices', 'dal13', '-c', 'os', '-k' 'test'])
+
+        self.assert_no_fail(result)
+        output = json.loads(result.output)
+        self.assertEqual(output[0][0]['Hourly'], 0.0)
+        self.assertEqual(output[0][1]['KeyName'], 'KeyName015')
+        self.assertEqual(output[0][1]['Price Id'], 1144)
+        self.assert_called_with('SoftLayer_Product_Package', 'getItemPrices')
+
     def test_package_list(self):
         p_mock = self.set_mock('SoftLayer_Product_Package', 'getAllObjects')
         p_mock.return_value = _get_all_packages()
@@ -101,12 +143,12 @@ class OrderTests(testing.TestCase):
 
         self.assert_no_fail(result)
         self.assert_called_with('SoftLayer_Product_Order', 'placeOrder')
-        self.assertEqual({'id': 1234,
-                          'created': order_date,
-                          'status': 'APPROVED'},
-                         json.loads(result.output))
+        self.assertNotIn('Warning: Closed soon: dal13', result.output)
+        self.assertIn('"status": "APPROVED"', result.output)
 
-    def test_place_with_quantity(self):
+    @mock.patch('SoftLayer.managers.network.NetworkManager.get_datacenter_by_keyname')
+    def test_place_with_quantity(self, keyname_mock):
+        keyname_mock.return_value = {"name": "ams01"}
         order_date = '2017-04-04 07:39:20'
         order = {'orderId': 1234, 'orderDate': order_date, 'placedOrder': {'status': 'APPROVED'}}
         verify_mock = self.set_mock('SoftLayer_Product_Order', 'verifyOrder')
@@ -117,15 +159,13 @@ class OrderTests(testing.TestCase):
         place_mock.return_value = order
         items_mock.return_value = self._get_order_items()
 
-        result = self.run_command(['-y', 'order', 'place', '--quantity=2', 'package', 'DALLAS13', 'ITEM1',
+        result = self.run_command(['-y', 'order', 'place', '--quantity=2', 'package', 'AMSTERDAM', 'ITEM1',
                                    '--complex-type', 'SoftLayer_Container_Product_Order_Thing'])
 
         self.assert_no_fail(result)
         self.assert_called_with('SoftLayer_Product_Order', 'placeOrder')
-        self.assertEqual({'id': 1234,
-                          'created': order_date,
-                          'status': 'APPROVED'},
-                         json.loads(result.output))
+        self.assertIn('Warning: Closed soon: ams01.pod01', result.output)
+        self.assertIn('"status": "APPROVED"', result.output)
 
     def test_place_extras_parameter_fail(self):
         result = self.run_command(['-y', 'order', 'place', 'package', 'DALLAS13', 'ITEM1',
@@ -225,15 +265,11 @@ class OrderTests(testing.TestCase):
                          json.loads(result.output))
 
     def test_preset_list(self):
-        active_preset1 = {'name': 'active1', 'keyName': 'PRESET1',
-                          'description': 'description1'}
-        active_preset2 = {'name': 'active2', 'keyName': 'PRESET2',
-                          'description': 'description2'}
-        acc_preset = {'name': 'account1', 'keyName': 'PRESET3',
-                      'description': 'description3'}
+        active_preset1 = {'name': 'active1', 'keyName': 'PRESET1', 'description': 'description1'}
+        active_preset2 = {'name': 'active2', 'keyName': 'PRESET2', 'description': 'description2'}
+        acc_preset = {'name': 'account1', 'keyName': 'PRESET3', 'description': 'description3'}
         active_mock = self.set_mock('SoftLayer_Product_Package', 'getActivePresets')
-        account_mock = self.set_mock('SoftLayer_Product_Package',
-                                     'getAccountRestrictedActivePresets')
+        account_mock = self.set_mock('SoftLayer_Product_Package', 'getAccountRestrictedActivePresets')
         active_mock.return_value = [active_preset1, active_preset2]
         account_mock.return_value = [acc_preset]
 
@@ -241,18 +277,10 @@ class OrderTests(testing.TestCase):
 
         self.assert_no_fail(result)
         self.assert_called_with('SoftLayer_Product_Package', 'getActivePresets')
-        self.assert_called_with('SoftLayer_Product_Package',
-                                'getAccountRestrictedActivePresets')
-        self.assertEqual([{'name': 'active1',
-                           'keyName': 'PRESET1',
-                           'description': 'description1'},
-                          {'name': 'active2',
-                           'keyName': 'PRESET2',
-                           'description': 'description2'},
-                          {'name': 'account1',
-                           'keyName': 'PRESET3',
-                           'description': 'description3'}],
-                         json.loads(result.output))
+        self.assert_called_with('SoftLayer_Product_Package', 'getAccountRestrictedActivePresets')
+        self.assertIn('"Key Name": "PRESET1",', result.output)
+        self.assertIn('"Description": "description2"', result.output)
+        self.assertIn('"Name": "account1",', result.output)
 
     def test_preset_list_keywork(self):
         result = self.run_command(['order', 'preset-list', 'package', '--keyword', 'testKeyWord'])
@@ -260,14 +288,30 @@ class OrderTests(testing.TestCase):
         self.assert_no_fail(result)
         self.assert_called_with('SoftLayer_Product_Package', 'getActivePresets', filter=_filter)
 
+    def test_preset_list_prices(self):
+        active_preset1 = {
+            'name': 'active1', 'keyName': 'PRESET1', 'description': 'description1', 'locations': [], 'id': 118822,
+            'prices': [
+                {'hourlyRecurringFee': 100, 'recurringFee': 55},
+                {'hourlyRecurringFee': 110, 'recurringFee': 44},
+            ]
+        }
+        active_mock = self.set_mock('SoftLayer_Product_Package', 'getActivePresets')
+        active_mock.return_value = [active_preset1]
+        result = self.run_command(['order', 'preset-list', 'package', '--prices'])
+        self.assert_no_fail(result)
+        self.assert_called_with('SoftLayer_Product_Package', 'getActivePresets')
+        self.assertIn("210", result.output)
+        self.assertIn("99", result.output)
+
     def test_location_list(self):
         result = self.run_command(['order', 'package-locations', 'package'])
         self.assert_no_fail(result)
         expected_results = [
-            {'id': 2017603, 'dc': 'wdc07', 'description': 'WDC07 - Washington, DC', 'keyName': 'WASHINGTON07'}
+            {'id': 2017603, 'dc': 'wdc07', 'description': 'WDC07 - Washington, DC',
+             'keyName': 'WASHINGTON07', 'Note': 'closed soon: wdc07.pod01'}
         ]
-        print("FUCK")
-        print(result.output)
+
         self.assertEqual(expected_results, json.loads(result.output))
 
     def test_quote_verify(self):
@@ -361,11 +405,21 @@ class OrderTests(testing.TestCase):
         self.assert_no_fail(result)
         self.assert_called_with('SoftLayer_Billing_Order_Quote', 'getObject', identifier='12345')
 
+    def test_quote_save(self):
+        result = self.run_command(['order', 'quote-save', '12345'])
+        self.assert_no_fail(result)
+        self.assert_called_with('SoftLayer_Billing_Order_Quote', 'saveQuote', identifier='12345')
+
     def test_quote_list(self):
         result = self.run_command(['order', 'quote-list'])
         self.assert_no_fail(result)
         self.assert_called_with('SoftLayer_Account', 'getActiveQuotes')
 
+    def test_cancelation(self):
+        result = self.run_command(['order', 'cancelation'])
+        self.assert_no_fail(result)
+        self.assert_called_with('SoftLayer_Billing_Item_Cancellation_Request', 'getAllCancellationRequests')
+
     def _get_order_items(self):
         item1 = {'keyName': 'ITEM1', 'description': 'description1',
                  'itemCategory': {'categoryCode': 'cat1'},
@@ -386,6 +440,18 @@ class OrderTests(testing.TestCase):
                   'recurringFee': '150'}
         return {'orderContainers': [{'prices': [price1, price2]}]}
 
+    def test_order_lookup(self):
+        result = self.run_command(['order', 'lookup', '12345'])
+        self.assert_no_fail(result)
+        self.assert_called_with('SoftLayer_Billing_Order', 'getObject', identifier='12345')
+        self.assertIn('Ordered By', result.output)
+        self.assertIn('Initial Invoice', result.output)
+
+    def test_quote_delete(self):
+        result = self.run_command(['order', 'quote-delete', '12345'])
+        self.assert_no_fail(result)
+        self.assert_called_with('SoftLayer_Billing_Order_Quote', 'deleteQuote', identifier='12345')
+
 
 def _get_all_packages():
     package_type = {'keyName': 'BARE_METAL_CPU'}
