@@ -7,9 +7,10 @@
 import json
 import sys
 
-import mock
+from unittest import mock as mock
 
 from SoftLayer.CLI import exceptions
+from SoftLayer.fixtures import SoftLayer_Product_Package
 from SoftLayer.fixtures import SoftLayer_Virtual_Guest as SoftLayer_Virtual_Guest
 from SoftLayer import SoftLayerAPIError
 from SoftLayer import SoftLayerError
@@ -143,19 +144,23 @@ class VirtTests(testing.TestCase):
         result = self.run_command(['vs', 'list', '--tag=tag'])
 
         self.assert_no_fail(result)
-        self.assertEqual(json.loads(result.output),
-                         [{'datacenter': 'TEST00',
-                           'primary_ip': '172.16.240.2',
-                           'hostname': 'vs-test1',
-                           'action': None,
-                           'id': 100,
-                           'backend_ip': '10.45.19.37'},
-                          {'datacenter': 'TEST00',
-                           'primary_ip': '172.16.240.7',
-                           'hostname': 'vs-test2',
-                           'action': None,
-                           'id': 104,
-                           'backend_ip': '10.45.19.35'}])
+
+    def test_list_vs_search_noargs(self):
+        result = self.run_command(['vs', 'list', '--search'])
+        self.assert_no_fail(result)
+        self.assert_called_with('SoftLayer_Search', 'advancedSearch', args=('_objectType:SoftLayer_Virtual_Guest ',))
+
+    def test_list_vs_search_noargs_domain(self):
+        result = self.run_command(['vs', 'list', '--search', '-Dtest'])
+        self.assert_no_fail(result)
+        self.assert_called_with('SoftLayer_Search', 'advancedSearch',
+                                args=('_objectType:SoftLayer_Virtual_Guest  domain: *test*',))
+
+    def test_list_vs_search_args(self):
+        result = self.run_command(['vs', 'list', '--search=thisTerm'])
+        self.assert_no_fail(result)
+        self.assert_called_with('SoftLayer_Search', 'advancedSearch',
+                                args=('_objectType:SoftLayer_Virtual_Guest *thisTerm*',))
 
     @mock.patch('SoftLayer.utils.lookup')
     def test_detail_vs_empty_billing(self, mock_lookup):
@@ -180,7 +185,7 @@ class VirtTests(testing.TestCase):
         self.assert_no_fail(result)
         output = json.loads(result.output)
         self.assertEqual(output['notes'], 'notes')
-        self.assertEqual(output['price_rate'], 6.54)
+        self.assertEqual(output['Price rate'], 1.54)
         self.assertEqual(output['users'][0]['username'], 'user')
         self.assertEqual(output['vlans'][0]['number'], 23)
         self.assertEqual(output['owner'], 'chechu')
@@ -216,6 +221,56 @@ class VirtTests(testing.TestCase):
             '-',
         )
 
+    def test_detail_drives_system(self):
+        mock = self.set_mock('SoftLayer_Virtual_Guest', 'getBlockDevices')
+        mock.return_value = [
+            {
+                "createDate": "2018-10-06T04:27:35-06:00",
+                "device": "0",
+                "id": 11111,
+                "mountType": "Disk",
+                "diskImage": {
+                    "capacity": 100,
+                    "description": "adns.vmware.com",
+                    "id": 72222,
+                    "name": "adns.vmware.com",
+                    "units": "GB",
+                }
+            }
+        ]
+        result = self.run_command(['vs', 'detail', '100'])
+
+        self.assert_no_fail(result)
+        output = json.loads(result.output)
+        self.assertEqual(output['drives'][0]['Capacity'], '100 GB')
+        self.assertEqual(output['drives'][0]['Name'], 'Disk')
+        self.assertEqual(output['drives'][0]['Type'], 'System')
+
+    def test_detail_drives_swap(self):
+        mock = self.set_mock('SoftLayer_Virtual_Guest', 'getBlockDevices')
+        mock.return_value = [
+            {
+                "device": "1",
+                "id": 22222,
+                "mountType": "Disk",
+                "statusId": 1,
+                "diskImage": {
+                    "capacity": 2,
+                    "description": "6211111-SWAP",
+                    "id": 33333,
+                    "name": "6211111-SWAP",
+                    "units": "GB",
+                }
+            }
+        ]
+        result = self.run_command(['vs', 'detail', '100'])
+
+        self.assert_no_fail(result)
+        output = json.loads(result.output)
+        self.assertEqual(output['drives'][0]['Capacity'], '2 GB')
+        self.assertEqual(output['drives'][0]['Name'], 'Disk')
+        self.assertEqual(output['drives'][0]['Type'], 'Swap')
+
     def test_detail_vs_dedicated_host_not_found(self):
         ex = SoftLayerAPIError('SoftLayer_Exception', 'Not found')
         mock = self.set_mock('SoftLayer_Virtual_DedicatedHost', 'getObject')
@@ -276,30 +331,35 @@ class VirtTests(testing.TestCase):
         output = json.loads(result.output)
         self.assertEqual(output.get('ptr', None), None)
 
+    def test_vs_detail_csv_output_format_with_nested_tables(self):
+        result = self.run_command(["--format", "csv", 'vs', 'detail', '100'])
+        result_output = result.output.replace('\r', '').split('\n')
+        self.assert_no_fail(result)
+        self.assertEqual(result_output[0], '"name","value"')
+        self.assertEqual(result_output[1], '"id",100')
+        self.assertEqual(result_output[16], '"drives","Type,Name,Drive,Capacity"')
+        self.assertEqual(result_output[17], '"drives","System,Disk,0,100 GB"')
+        self.assertEqual(result_output[18], '"drives","Swap,Disk,1,2 GB"')
+        self.assertEqual(result_output[30], '"vlans","type,number,id"')
+        self.assertEqual(result_output[31], '"vlans","PUBLIC,23,1"')
+        self.assertEqual(result_output[32], '"Bandwidth","Type,In GB,Out GB,Allotment"')
+        self.assertEqual(result_output[33], '"Bandwidth","Public,.448,.52157,250"')
+        self.assertEqual(result_output[34], '"Bandwidth","Private,.03842,.01822,N/A"')
+        self.assertEqual(result_output[35], '"security_groups","interface,id,name"')
+        self.assertEqual(result_output[36], '"security_groups","PRIVATE,128321,allow_all"')
+
     def test_create_options(self):
-        result = self.run_command(['vs', 'create-options'])
+        result = self.run_command(['vs', 'create-options', '--vsi-type', 'TRANSIENT_CLOUD_SERVER'])
+        self.assert_no_fail(result)
+
+    def test_create_options_prices(self):
+        result = self.run_command(['vs', 'create-options', '--prices', '--vsi-type', 'TRANSIENT_CLOUD_SERVER'])
+        self.assert_no_fail(result)
 
+    def test_create_options_prices_location(self):
+        result = self.run_command(['vs', 'create-options', '--prices', 'dal13',
+                                   '--vsi-type', 'TRANSIENT_CLOUD_SERVER'])
         self.assert_no_fail(result)
-        self.assertEqual({'cpus (dedicated host)': [4, 56],
-                          'cpus (dedicated)': [1],
-                          'cpus (standard)': [1, 2, 3, 4],
-                          'datacenter': ['ams01', 'dal05'],
-                          'flavors (balanced)': ['B1_1X2X25', 'B1_1X2X100'],
-                          'flavors (balanced local - hdd)': ['BL1_1X2X100'],
-                          'flavors (balanced local - ssd)': ['BL2_1X2X100'],
-                          'flavors (compute)': ['C1_1X2X25'],
-                          'flavors (memory)': ['M1_1X2X100'],
-                          'flavors (GPU)': ['AC1_1X2X100', 'ACL1_1X2X100'],
-                          'flavors (transient)': ['B1_1X2X25_TRANSIENT'],
-                          'local disk(0)': ['25', '100'],
-                          'memory': [1024, 2048, 3072, 4096],
-                          'memory (dedicated host)': [8192, 65536],
-                          'nic': ['10', '100', '1000'],
-                          'nic (dedicated host)': ['1000'],
-                          'os (CENTOS)': 'CENTOS_6_64',
-                          'os (DEBIAN)': 'DEBIAN_7_64',
-                          'os (UBUNTU)': 'UBUNTU_12_64'},
-                         json.loads(result.output))
 
     @mock.patch('SoftLayer.CLI.formatting.confirm')
     def test_dns_sync_both(self, confirm_mock):
@@ -320,19 +380,19 @@ class VirtTests(testing.TestCase):
                                            'getResourceRecords')
         getResourceRecords.return_value = []
         createAargs = ({
-            'type': 'a',
-            'host': 'vs-test1',
-            'domainId': 12345,  # from SoftLayer_Account::getDomains
-            'data': '172.16.240.2',
-            'ttl': 7200
-        },)
+                           'type': 'a',
+                           'host': 'vs-test1',
+                           'domainId': 12345,  # from SoftLayer_Account::getDomains
+                           'data': '172.16.240.2',
+                           'ttl': 7200
+                       },)
         createPTRargs = ({
-            'type': 'ptr',
-            'host': '2',
-            'domainId': 123456,
-            'data': 'vs-test1.test.sftlyr.ws',
-            'ttl': 7200
-        },)
+                             'type': 'ptr',
+                             'host': '2',
+                             'domainId': 123456,
+                             'data': 'vs-test1.test.sftlyr.ws',
+                             'ttl': 7200
+                         },)
 
         result = self.run_command(['vs', 'dns-sync', '100'])
 
@@ -375,12 +435,12 @@ class VirtTests(testing.TestCase):
             }
         }
         createV6args = ({
-            'type': 'aaaa',
-            'host': 'vs-test1',
-            'domainId': 12345,
-            'data': '2607:f0d0:1b01:0023:0000:0000:0000:0004',
-            'ttl': 7200
-        },)
+                            'type': 'aaaa',
+                            'host': 'vs-test1',
+                            'domainId': 12345,
+                            'data': '2607:f0d0:1b01:0023:0000:0000:0000:0004',
+                            'ttl': 7200
+                        },)
         guest.return_value = test_guest
         result = self.run_command(['vs', 'dns-sync', '--aaaa-record', '100'])
         self.assert_no_fail(result)
@@ -520,6 +580,29 @@ class VirtTests(testing.TestCase):
         self.assertIn({'id': 1122}, order_container['prices'])
         self.assertEqual(order_container['virtualGuests'], [{'id': 100}])
 
+    @mock.patch('SoftLayer.CLI.formatting.confirm')
+    def test_upgrade_disk(self, confirm_mock):
+        confirm_mock.return_value = True
+        mock = self.set_mock('SoftLayer_Product_Package', 'getItemPrices')
+        mock.return_value = SoftLayer_Product_Package.getVSDiskItemPrices
+        result = self.run_command(['vs', 'upgrade', '100', '--flavor=M1_64X512X100',
+                                   '--resize-disk=10', '1', '--resize-disk=10', '2'])
+        self.assert_no_fail(result)
+        self.assert_called_with('SoftLayer_Product_Order', 'placeOrder')
+        call = self.calls('SoftLayer_Product_Order', 'placeOrder')[0]
+        order_container = call.args[0]
+        self.assertEqual(799, order_container['presetId'])
+        self.assertIn({'id': 100}, order_container['virtualGuests'])
+        self.assertEqual(order_container['virtualGuests'], [{'id': 100}])
+
+    @mock.patch('SoftLayer.CLI.formatting.confirm')
+    def test_upgrade_disk_error(self, confirm_mock):
+        confirm_mock.return_value = True
+        result = self.run_command(['vs', 'upgrade', '100', '--flavor=M1_64X512X100',
+                                   '--resize-disk=1000', '1', '--resize-disk=10', '2'])
+        self.assertEqual(result.exit_code, 1)
+        self.assertIsInstance(result.exception, SoftLayerAPIError)
+
     @mock.patch('SoftLayer.CLI.formatting.confirm')
     def test_upgrade_with_flavor(self, confirm_mock):
         confirm_mock.return_value = True
@@ -532,6 +615,19 @@ class VirtTests(testing.TestCase):
         self.assertIn({'id': 100}, order_container['virtualGuests'])
         self.assertEqual(order_container['virtualGuests'], [{'id': 100}])
 
+    @mock.patch('SoftLayer.CLI.formatting.confirm')
+    def test_upgrade_with_add_disk(self, confirm_mock):
+        confirm_mock.return_value = True
+        mock = self.set_mock('SoftLayer_Product_Package', 'getItemPrices')
+        mock.return_value = SoftLayer_Product_Package.getVSDiskItemPrices
+        result = self.run_command(['vs', 'upgrade', '100', '--add-disk=10', '--add-disk=10'])
+        self.assert_no_fail(result)
+        self.assert_called_with('SoftLayer_Product_Order', 'placeOrder')
+        call = self.calls('SoftLayer_Product_Order', 'placeOrder')[0]
+        order_container = call.args[0]
+        self.assertIn({'id': 100}, order_container['virtualGuests'])
+        self.assertEqual(order_container['virtualGuests'], [{'id': 100}])
+
     @mock.patch('SoftLayer.CLI.formatting.confirm')
     def test_upgrade_with_cpu_memory_and_flavor(self, confirm_mock):
         confirm_mock.return_value = True
@@ -552,7 +648,13 @@ class VirtTests(testing.TestCase):
                                    '100'])
 
         self.assert_no_fail(result)
-        self.assertEqual(result.output, '')
+        expected_output = '"The userdata of virtual server instance: 100 was updated."\n' \
+                          + '"The hostname of virtual server instance: 100 was updated."\n' \
+                          + '"The domain of virtual server instance: 100 was updated."\n' \
+                          + '"The tags of virtual server instance: 100 was updated."\n' \
+                          + '"The public speed of virtual server instance: 100 was updated."\n' \
+                          + '"The private speed of virtual server instance: 100 was updated."\n'
+        self.assertEqual(result.output, expected_output)
 
         self.assert_called_with(
             'SoftLayer_Virtual_Guest', 'editObject',
@@ -658,7 +760,7 @@ class VirtTests(testing.TestCase):
 
         result = self.run_command(['vs', 'capture', '100', '--name', 'TestName'])
         self.assert_no_fail(result)
-        self.assert_called_with('SoftLayer_Virtual_Guest', 'createArchiveTransaction', identifier=100)
+        self.assert_called_with('SoftLayer_Virtual_Guest', 'createArchiveTemplate', identifier=100)
 
     @mock.patch('SoftLayer.CLI.formatting.no_going_back')
     def test_usage_no_confirm(self, confirm_mock):
@@ -679,6 +781,13 @@ class VirtTests(testing.TestCase):
 
         self.assert_no_fail(result)
 
+    def test_usage_vs_cpu_lower_case(self):
+        result = self.run_command(
+            ['vs', 'usage', '100', '--start_date=2019-3-4', '--end_date=2019-4-2', '--valid_type=cpu0',
+             '--summary_period=300'])
+
+        self.assert_no_fail(result)
+
     def test_usage_vs_memory(self):
         result = self.run_command(
             ['vs', 'usage', '100', '--start_date=2019-3-4', '--end_date=2019-4-2', '--valid_type=MEMORY_USAGE',
@@ -697,6 +806,7 @@ class VirtTests(testing.TestCase):
         self.assertIsInstance(result.exception, exceptions.CLIAbort)
 
     def test_bandwidth_vs(self):
+        self.skipTest("Skipping until VIRT-11733 is released")
         if sys.version_info < (3, 6):
             self.skipTest("Test requires python 3.6+")
 
@@ -725,6 +835,7 @@ class VirtTests(testing.TestCase):
         self.assertEqual(output_list[0]['Pub In'], 1.3503)
 
     def test_bandwidth_vs_quite(self):
+        self.skipTest("Skipping until VIRT-11733 is released")
         result = self.run_command(['vs', 'bandwidth', '100', '--start_date=2019-01-01', '--end_date=2019-02-01', '-q'])
         self.assert_no_fail(result)
 
@@ -740,3 +851,166 @@ class VirtTests(testing.TestCase):
         self.assertEqual(output_summary[1]['Max Date'], date)
         self.assertEqual(output_summary[2]['Max GB'], 0.1172)
         self.assertEqual(output_summary[3]['Sum GB'], 0.0009)
+
+    def test_vs_storage(self):
+        result = self.run_command(
+            ['vs', 'storage', '100'])
+
+        self.assert_no_fail(result)
+
+    def test_billing(self):
+        result = self.run_command(['vs', 'billing', '123456'])
+        vir_billing = {
+            'Billing Item Id': 6327,
+            'Id': '123456',
+            'Provision Date': None,
+            'Recurring Fee': None,
+            'Total': 1.54,
+            'prices': [
+                {'Description': '1 GB', 'Recurring Price': 1},
+                {'Description': 'Reboot / Remote Console', 'Recurring Price': 1},
+                {'Description': '1 Gbps Public & Private Network Uplinks', 'Recurring Price': 1},
+                {'Description': '1 Gbps Public Uplink', 'Recurring Price': 1},
+                {'Description': '1 Gbps Private Uplink', 'Recurring Price': 1}
+            ]
+        }
+        self.assert_no_fail(result)
+        self.assertEqual(json.loads(result.output), vir_billing)
+
+    def test_vs_migrate_list(self):
+        result = self.run_command(['vs', 'migrate'])
+        self.assert_no_fail(result)
+        self.assert_called_with('SoftLayer_Account', 'getVirtualGuests')
+        self.assert_not_called_with('SoftLayer_Virtual_Guest', 'migrate')
+        self.assert_not_called_with('SoftLayer_Virtual_Guest', 'migrateDedicatedHost')
+
+    def test_vs_migrate_list_empty(self):
+        mock = self.set_mock('SoftLayer_Account', 'getVirtualGuests')
+        mock.return_value = []
+        result = self.run_command(['vs', 'migrate'])
+        self.assert_no_fail(result)
+        self.assert_called_with('SoftLayer_Account', 'getVirtualGuests')
+        self.assert_not_called_with('SoftLayer_Virtual_Guest', 'migrate')
+        self.assert_not_called_with('SoftLayer_Virtual_Guest', 'migrateDedicatedHost')
+        self.assertIn("No guests require migration at this time", result.output)
+
+    def test_vs_migrate_guest(self):
+        result = self.run_command(['vs', 'migrate', '-g', '100'])
+        self.assert_no_fail(result)
+        self.assertIn('Started a migration on', result.output)
+        self.assert_not_called_with('SoftLayer_Account', 'getVirtualGuests')
+        self.assert_called_with('SoftLayer_Virtual_Guest', 'migrate', identifier=100)
+        self.assert_not_called_with('SoftLayer_Virtual_Guest', 'migrateDedicatedHost')
+
+    def test_vs_migrate_all(self):
+        result = self.run_command(['vs', 'migrate', '-a'])
+        self.assert_no_fail(result)
+        self.assertIn('Started a migration on', result.output)
+        self.assert_called_with('SoftLayer_Virtual_Guest', 'migrate', identifier=100)
+        self.assert_called_with('SoftLayer_Virtual_Guest', 'migrate', identifier=104)
+        self.assert_not_called_with('SoftLayer_Virtual_Guest', 'migrateDedicatedHost')
+
+    def test_vs_migrate_all_empty(self):
+        mock = self.set_mock('SoftLayer_Account', 'getVirtualGuests')
+        mock.return_value = []
+        result = self.run_command(['vs', 'migrate', '-a'])
+        self.assert_no_fail(result)
+        self.assertIn('No guests require migration at this time', result.output)
+
+    def test_vs_migrate_dedicated(self):
+        result = self.run_command(['vs', 'migrate', '-g', '100', '-h', '999'])
+        self.assert_no_fail(result)
+        self.assertIn('Started a migration on', result.output)
+        self.assert_not_called_with('SoftLayer_Account', 'getVirtualGuests')
+        self.assert_not_called_with('SoftLayer_Virtual_Guest', 'migrate', identifier=100)
+        self.assert_not_called_with('SoftLayer_Virtual_Guest', 'migrateDedicatedHost', args=(999), identifier=100)
+
+    def test_vs_migrate_exception(self):
+        ex = SoftLayerAPIError('SoftLayer_Exception', 'PROBLEM')
+        mock = self.set_mock('SoftLayer_Virtual_Guest', 'migrate')
+        mock.side_effect = ex
+        result = self.run_command(['vs', 'migrate', '-g', '100'])
+        self.assert_no_fail(result)
+        self.assertIn('Failed to migrate', result.output)
+        self.assert_not_called_with('SoftLayer_Account', 'getVirtualGuests')
+        self.assert_called_with('SoftLayer_Virtual_Guest', 'migrate', identifier=100)
+        self.assert_not_called_with('SoftLayer_Virtual_Guest', 'migrateDedicatedHost', args=(999), identifier=100)
+
+    def test_credentail(self):
+        result = self.run_command(['vs', 'credentials', '100'])
+        self.assert_no_fail(result)
+        # self.assertEqual(json.loads(result.output), [{
+        #     "username": "user",
+        #     "password": "pass"
+        # }])
+
+    @mock.patch('SoftLayer.CLI.formatting.confirm')
+    def test_authorize_storage_vs_no_confirm(self, confirm_mock):
+        confirm_mock.return_value = False
+        result = self.run_command(['vs', 'authorize-storage', '-u', '1234'])
+
+        self.assertEqual(result.exit_code, 2)
+
+    @mock.patch('SoftLayer.CLI.formatting.confirm')
+    def test_authorize_vs_empty(self, confirm_mock):
+        confirm_mock.return_value = True
+        storage_result = self.set_mock('SoftLayer_Account', 'getNetworkStorage')
+        storage_result.return_value = []
+        result = self.run_command(['vs', 'authorize-storage', '--username-storage=#', '1234'])
+
+        self.assertEqual(str(result.exception), "The Storage with username: # was not found, "
+                                                "please enter a valid storage username")
+
+    def test_authorize_storage_vs(self):
+        result = self.run_command(['vs', 'authorize-storage', '--username-storage=SL01SEL301234-11', '1234'])
+        self.assert_no_fail(result)
+
+    def test_authorize_portable_storage_vs(self):
+        mock = self.set_mock('SoftLayer_Virtual_Guest', 'attachDiskImage')
+        mock.return_value = {
+            "createDate": "2021-03-22T13:15:31-06:00",
+            "id": 1234567
+        }
+        result = self.run_command(['vs', 'authorize-storage', '--portable-id=12345', '1234'])
+        self.assert_no_fail(result)
+
+    def test_authorize_volume_and_portable_storage_vs(self):
+        result = self.run_command(['vs', 'authorize-storage', '--username-storage=SL01SEL301234-11',
+                                   '--portable-id=12345', '1234'])
+        self.assert_no_fail(result)
+
+    def test_monitoring_vs(self):
+        result = self.run_command(['vs', 'monitoring', '1234'])
+        self.assert_no_fail(result)
+
+    def test_last_transaction_empty(self):
+        vg_return = SoftLayer_Virtual_Guest.getObject
+        transaction = []
+
+        vg_return['lastTransaction'] = transaction
+        mock = self.set_mock('SoftLayer_Virtual_Guest', 'getObject')
+        mock.return_value = vg_return
+        result = self.run_command(['vs', 'detail', '100'])
+        self.assert_no_fail(result)
+
+    def test_user_access(self):
+        result = self.run_command(['vs', 'access', '100'])
+        self.assert_no_fail(result)
+
+    def test_notifications(self):
+        result = self.run_command(['vs', 'notifications', '100'])
+        self.assert_no_fail(result)
+
+    def test_add_notification(self):
+        result = self.run_command(['vs', 'notification-add', '100', '--users', '123456'])
+        self.assert_no_fail(result)
+
+    def test_notification_delete(self):
+        result = self.run_command(['vs', 'notification-delete', '100'])
+        self.assert_no_fail(result)
+
+    def test_os_available(self):
+        _mock = self.set_mock('SoftLayer_Product_Package', 'getItems')
+        _mock.return_value = SoftLayer_Product_Package.getItemsOS
+        result = self.run_command(['vs', 'os-available'])
+        self.assert_no_fail(result)
