@@ -4,9 +4,10 @@
 
     :license: MIT, see LICENSE for more details.
 """
-import mock
+
 import sys
 import unittest
+from unittest import mock as mock
 
 import SoftLayer
 from SoftLayer import fixtures
@@ -100,6 +101,14 @@ class NetworkTests(testing.TestCase):
 
         self.assertEqual(fixtures.SoftLayer_Product_Order.verifyOrder, result)
 
+        result = self.network.add_subnet('static',
+                                         quantity=8,
+                                         endpoint_id=1234,
+                                         version=4,
+                                         test_order=True)
+
+        self.assertEqual(fixtures.SoftLayer_Product_Order.verifyOrder, result)
+
     def test_add_subnet_for_ipv6(self):
         # Test a public IPv6 order
         result = self.network.add_subnet('public',
@@ -158,6 +167,26 @@ class NetworkTests(testing.TestCase):
         self.assert_called_with('SoftLayer_Billing_Item', 'cancelService',
                                 identifier=1056)
 
+    def test_set_tags_subnet(self):
+        subnet_id = 1234
+        tags = 'tags1,tag2'
+        result = self.network.set_tags_subnet(subnet_id, tags)
+
+        self.assertEqual(result, True)
+        self.assert_called_with('SoftLayer_Network_Subnet', 'setTags',
+                                identifier=subnet_id,
+                                args=(tags,))
+
+    def test_edit_note_subnet(self):
+        subnet_id = 1234
+        note = 'test note'
+        result = self.network.edit_note_subnet(subnet_id, note)
+
+        self.assertEqual(result, True)
+        self.assert_called_with('SoftLayer_Network_Subnet', 'editNote',
+                                identifier=subnet_id,
+                                args=(note,))
+
     def test_create_securitygroup(self):
         result = self.network.create_securitygroup(name='foo',
                                                    description='bar')
@@ -194,39 +223,6 @@ class NetworkTests(testing.TestCase):
                                 'detachNetworkComponents',
                                 identifier=100, args=([500, 600],))
 
-    def test_edit_rwhois(self):
-        result = self.network.edit_rwhois(
-            abuse_email='abuse@test.foo',
-            address1='123 Test Street',
-            address2='Apt. #31',
-            city='Anywhere',
-            company_name='TestLayer',
-            country='US',
-            first_name='Bob',
-            last_name='Bobinson',
-            postal_code='9ba62',
-            private_residence=False,
-            state='TX')
-
-        self.assertEqual(result, True)
-        expected = {
-            'abuseEmail': 'abuse@test.foo',
-            'address1': '123 Test Street',
-            'address2': 'Apt. #31',
-            'city': 'Anywhere',
-            'companyName': 'TestLayer',
-            'country': 'US',
-            'firstName': 'Bob',
-            'lastName': 'Bobinson',
-            'postalCode': '9ba62',
-            'privateResidenceFlag': False,
-            'state': 'TX',
-        }
-        self.assert_called_with('SoftLayer_Network_Subnet_Rwhois_Data',
-                                'editObject',
-                                identifier='id',
-                                args=(expected,))
-
     def test_edit_securitygroup(self):
         result = self.network.edit_securitygroup(100, name='foobar')
 
@@ -262,12 +258,6 @@ class NetworkTests(testing.TestCase):
                                         'portRangeMin': -1, 'portRangeMax': -1,
                                         'ethertype': '', 'remoteIp': ''}],))
 
-    def test_get_rwhois(self):
-        result = self.network.get_rwhois()
-
-        self.assertEqual(result, fixtures.SoftLayer_Account.getRwhoisData)
-        self.assert_called_with('SoftLayer_Account', 'getRwhoisData')
-
     def test_get_securitygroup(self):
         result = self.network.get_securitygroup(100)
 
@@ -320,15 +310,6 @@ class NetworkTests(testing.TestCase):
                                 filter=_filter)
 
     def test_list_subnets_default(self):
-        result = self.network.list_subnets()
-
-        self.assertEqual(result, fixtures.SoftLayer_Account.getSubnets)
-        _filter = {'subnets': {'subnetType': {'operation': '!= GLOBAL_IP'}}}
-        self.assert_called_with('SoftLayer_Account', 'getSubnets',
-                                mask='mask[%s]' % network.DEFAULT_SUBNET_MASK,
-                                filter=_filter)
-
-    def test_list_subnets_with_filters(self):
         result = self.network.list_subnets(
             identifier='10.0.0.1',
             datacenter='dal00',
@@ -336,7 +317,6 @@ class NetworkTests(testing.TestCase):
             subnet_type='PRIMARY',
             network_space='PUBLIC',
         )
-
         self.assertEqual(result, fixtures.SoftLayer_Account.getSubnets)
         _filter = {
             'subnets': {
@@ -354,6 +334,14 @@ class NetworkTests(testing.TestCase):
                                 mask='mask[%s]' % network.DEFAULT_SUBNET_MASK,
                                 filter=_filter)
 
+    def test_list_subnets_with_filters(self):
+        result = self.network.list_subnets()
+
+        self.assertEqual(result, fixtures.SoftLayer_Account.getSubnets)
+
+        self.assert_called_with('SoftLayer_Account', 'getSubnets',
+                                mask='mask[%s]' % network.DEFAULT_SUBNET_MASK)
+
     def test_list_vlans_default(self):
         result = self.network.list_vlans()
 
@@ -370,13 +358,14 @@ class NetworkTests(testing.TestCase):
         self.assertEqual(result, fixtures.SoftLayer_Account.getNetworkVlans)
         _filter = {
             'networkVlans': {
-                'primaryRouter': {
-                    'datacenter': {
-                        'name': {'operation': '_= dal00'}},
-                },
+                'id': {
+                    'operation': 'orderBy',
+                    'options': [
+                        {'name': 'sort', 'value': ['ASC']}]},
                 'vlanNumber': {'operation': 5},
                 'name': {'operation': '_= primary-vlan'},
-            },
+                'primaryRouter': {
+                    'datacenter': {'name': {'operation': '_= dal00'}}}}
         }
         self.assert_called_with('SoftLayer_Account', 'getNetworkVlans',
                                 filter=_filter)
@@ -625,3 +614,46 @@ class NetworkTests(testing.TestCase):
         _filter = {'objectName': {'operation': 'CCI'}}
         self.assert_called_with('SoftLayer_Event_Log', 'getAllObjects', filter=_filter)
         self.assertEqual(100, log['accountId'])
+
+    def test_vlan_edit(self):
+        vlan_id = 100
+        name = "test"
+        note = "test note"
+        tags = "tag1,tag2"
+
+        self.network.edit(vlan_id, name, note, tags)
+        self.assert_called_with('SoftLayer_Network_Vlan', 'editObject')
+
+    def test_get_all_pods(self):
+        self.network.get_pods()
+        self.assert_called_with('SoftLayer_Network_Pod', 'getAllObjects')
+
+    def test_route(self):
+        self.network.route('SoftLayer_Hardware_Server', 123456, 100)
+        self.assert_called_with('SoftLayer_Network_Subnet', 'route')
+
+    def test_get_all_datacenter(self):
+        self.network.get_datacenter()
+        self.assert_called_with('SoftLayer_Location', 'getDatacenters')
+        self.network.get_datacenter(datacenter="dal11")
+        expected_filter = {"name": {"operation": "dal11"}}
+        self.assert_called_with('SoftLayer_Location', 'getDatacenters', filter=expected_filter)
+
+    def test_get_datacenter_by_keyname(self):
+        # normal operation
+        result = self.network.get_datacenter_by_keyname("TEST01")
+        expected_filter = {"regions": {"keyname": {"operation": "TEST01"}}}
+        self.assert_called_with('SoftLayer_Location', 'getDatacenters', filter=expected_filter)
+        self.assertEqual(result.get('name'), 'dal13')
+
+        # an "empty" result
+        SoftLayer_Location = self.set_mock('SoftLayer_Location', 'getDatacenters')
+        SoftLayer_Location.return_value = []
+        result = self.network.get_datacenter_by_keyname("TEST01")
+        self.assertEqual(result, {})
+
+    def test_clear_route(self):
+        result = self.network.clear_route(1234)
+
+        self.assertEqual(result, True)
+        self.assert_called_with('SoftLayer_Network_Subnet', 'clearRoute')
